# Generated by roxygen2: do not edit by hand

S3method(plot,conceptMap)
S3method(print,conceptMap)
S3method(summary,conceptMap)
export(abbreviateStatements)
export(checkConceptMapData)
export(conceptMapping)
export(createStatementOverview)
export(crossClusterMap)
export(numberOfSorters)
export(numberOfStatements)
export(printSorters)
export(selectStatements)
export(simulateCardData)
export(sorterMapping)
import(clue)
import(ggplot2)
importFrom(cluster,silhouette)
importFrom(factoextra,fviz_cluster)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(igraph,V)
importFrom(igraph,as_data_frame)
importFrom(igraph,cluster_leiden)
importFrom(igraph,gorder)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_with_fr)
importFrom(igraph,strength)
importFrom(pheatmap,pheatmap)
importFrom(stats,chisq.test)
importFrom(stats,cmdscale)
importFrom(stats,dist)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_squish)
importFrom(stringr,str_to_lower)
